@@ -2,14 +2,14 @@ import aurorax
 from typing import Dict, List
 
 
-def validate(schema: List, record: Dict, quiet: bool = False) -> bool:
+def validate(schema: List[Dict], record: Dict, quiet: bool = False) -> bool:
     """
     Validate the metadata record against a schema. This checks
     that the key names match and there aren't fewer or more keys
     than expected.
 
     :param schema: metadata schema to validate against
-    :type schema: List
+    :type schema: List[Dict]
     :param record: metadata record to validate
     :type record: Dict
 
@@ -26,7 +26,7 @@ def validate(schema: List, record: Dict, quiet: bool = False) -> bool:
     return True
 
 
-def get_ephemeris_schema(identifier: int) -> List:
+def get_ephemeris_schema(identifier: int) -> List[Dict]:
     """
     Retrieve the metadata schema for an ephemeris record
 
@@ -34,16 +34,16 @@ def get_ephemeris_schema(identifier: int) -> List:
     :type identifier: int
 
     :return: metadata schema
-    :rtype: List
+    :rtype: List[Dict]
     """
     source_info = aurorax.sources.get_using_identifier(identifier, format="full_record")
-    if ("ephemeris_metadata_schema" in source_info):
-        return source_info["ephemeris_metadata_schema"]
+    if source_info.ephemeris_metadata_schema:
+        return source_info.ephemeris_metadata_schema
     else:
         return []
 
 
-def get_data_products_schema(identifier: int) -> List:
+def get_data_products_schema(identifier: int) -> List[Dict]:
     """
     Retrieve the metadata schema for a data products record
 
@@ -51,10 +51,10 @@ def get_data_products_schema(identifier: int) -> List:
     :type identifier: int
 
     :return: metadata schema
-    :rtype: List
+    :rtype: List[Dict]
     """
     source_info = aurorax.sources.get_using_identifier(identifier, format="full_record")
-    if ("data_product_metadata_schema" in source_info):
-        return source_info["data_product_metadata_schema"]
+    if source_info.data_product_metadata_schema:
+        return source_info.data_product_metadata_schema
     else:
         return []
